`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineer: Samuel Lowe
// 
// Create Date: 7/11/2015 
// Design Name: Nexys Video Looper project OLED component
// Module Name: XADCdemo 
// Target Devices: Digilent Nexys Video rev.A
// Tool Versions: Vivado 2015.1
// Description: will display all neccesary information on the
//              onboard OLED display
// Dependencies: 
// 
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: One issue is that for reliable results on the OLED, it may be 
//               neccesary to cycle power on the board  before reprogramming
// 
//////////////////////////////////////////////////////////////////////////////////
 

module XADCdemo(
    input CLK100MHZ,
    //from Looper
    input current_track,
    input [3:0] current_bank,
    //input [7:0] recording,
    //input [7:0] playing,
    //input [31:0] max_address,
    //input [31:0] current_address,
       
   //OLED signals
    output oled_dc,
    output oled_res,
    output oled_sclk,
    output oled_sdin,
    output oled_vbat,
    output oled_vdd
 );
   
   reg [3:0] decimal;   
   
   reg [32:0] max_address = 300000000;
   reg [32:0] current_address = 0;
   
   always @(posedge (CLK100MHZ))begin
        if(current_address <= max_address)
            current_address <= current_address + 1;
        else
            current_address <= 0;   
   end
   
   
    reg [7:0] bank_dig0 = 0;
    reg [7:0] bank_dig1 = 0;
                     
     ///////////////////////////////////////////////////////////////////
     //binary to decimal conversion
     //////////////////////////////////////////////////////////////////
     
      always @ (posedge(CLK100MHZ)) //when data is ready to be read from register
      begin
        
        decimal <= current_bank;
        //it looks nicer if our max value is 1V instead of .999755

        bank_dig0 = decimal % 10;
        decimal = decimal / 10;
        
        bank_dig1 = decimal % 10;                       
                   
      end
           

     ///////////////////////////////////////////////////////////////////
     //binary to decimal conversion
     //////////////////////////////////////////////////////////////////
           
       PmodOLEDCtrl OLED(
                        .CLK(CLK100MHZ),
                        .CS(),
                        .SDIN(oled_sdin),
                        .SCLK(oled_sclk),
                        .DC(oled_dc),
                        .RES(oled_res),
                        .VBAT(oled_vbat),
                        .VDD(oled_vdd),   
                        
                        .current_track(current_track),
                        .record_reg(),
                        .play_reg(),
                        .max_address(max_address),
                        .current_address(current_address),
                        .bank_dig0(bank_dig0),
                        .bank_dig1(bank_dig1)
                        
       );
       
endmodule
